# Can I invoke ZSH in a subprocess?

import subprocess

# Implements the full command
subprocess.run(['ls', '-al'])

# Does ls but no options
print()
subprocess.run(['ls', '-al'], shell=True)

# Gives "zsh: can't open input file: ls"
print()
subprocess.run(['zsh', 'ls', '-al'])

# zsh: can't open input file: "ls -al"
print()
subprocess.run(['zsh', '"ls -al"'])

# Another error
print()
subprocess.run("zsh ls -al", shell=True)

# Works as expected
print()
subprocess.run("ls -al", shell=True)

# && works
print()
subprocess.run("ls -al && ls -al", shell=True)

# !$ doesn't work - we're in /bin/sh
print()
subprocess.run("ls -al && echo \"!$\"", shell=True)

# So this doesn't work either
print()
subprocess.run("l", shell=True)

# This calls ZSH, but doesn't get .zshrc
print()
subprocess.run("l", shell=True, executable="/bin/zsh")

# Still doesn't work
print()
subprocess.run("ls -al && echo \"!$\"", shell=True, executable="/bin/zsh")

# Back to sh, try our own command - this works
print()
subprocess.run("busy list", shell=True)

# But it also works in a list? Yes
print()
subprocess.run(["busy", "list"])

# But now we can't use &&, so shell is better
print()
subprocess.run(["busy", "list", "&&", "busy", "list"])

# Can we get variables from .zshrc? Maybe if already set
print()
subprocess.run("echo $EDITOR", shell=True)



import curses
from busy.util.textbox_util import NewTextbox
from curses.textpad import rectangle

# x and y represent the top corner of the rectangle. The height of the rectangle
# is always 3, so the height of the line is always 1. It always extends to the
# right edge of the parent window.

def one_line_textbox(win):
    y, x = win.getyx()
    h, w = win.getmaxyx()
    rectangle(win, y-1, x+1, y+1, w-x)
    win.refresh()
    win2 = curses.newwin(1, w-x-8, y, x+2)
    box = NewTextbox(win2)
    result = box.edit()
    # win2.getch()

def tester(fullwin):
    fullwin.move(4, 0)
    fullwin.addstr("Value:")
    result = one_line_textbox(fullwin)

curses.wrapper(tester)


from busy.ui.ui import UI
from enum import Enum
E = Enum('E', ['YES','NO'])
c1 = UI.Chooser.Choice(keys=['y'], commands=['yes'], action=E.YES)
c2 = UI.Chooser.Choice(keys=['n'], commands=['no'], action=E.NO)
c = UI.Chooser([c1, c2])



# Can I override stdin or stdout for readline/input? I guess not.

import readline
from contextlib import redirect_stdout
from io import StringIO


with redirect_stdout(StringIO()) as f:
    print("Hello")
    readline.parse_and_bind('tab: complete')
    input()
print(f.getvalue())


class SuperWrapper:
    @classmethod
    def wrap(parent_class, method):
        def wrapper(self, *args, **kwargs):
            parent_method = getattr(parent_class, method.__name__)
            parent_method(self, method, *args, **kwargs)
        return wrapper


class Parent(SuperWrapper):

    def execute(self, method, *args, **kwargs):
        print(f"Parent execute before")
        method(self, *args, **kwargs)
        print(f"Parent execute after")


class InBetween(Parent):

    @Parent.wrap
    def execute(self, method, *args, **kwargs):
        print(f"IB execute before")
        method(self, *args, **kwargs)
        print(f"IB execute after")

class NewChild(InBetween):

    @InBetween.wrap
    def execute(self, name):
        print(f"Hello {name}")

c = NewChild()
c.execute("Jane")




from dataclasses import dataclass

@dataclass(kw_only=True)
class Item:
    """Class for keeping track of an item in inventory."""
    name: str
    unit_price: float
    quantity_on_hand: int = 0

    def total_cost(self) -> float:
        return self.unit_price * self.quantity_on_hand

@dataclass
class J(Item):
    description: str
